@@ -1,7 +1,8 @@
-#! /usr/bin/python3
+#!/usr/bin/python3
 
 from gi.repository import Gio, GObject, GLib, CScreensaver
 
+import config
 import re
 import os
 import signal
@@ -19,7 +20,8 @@ class AuthClient(GObject.Object):
         'auth-failure': (GObject.SignalFlags.RUN_LAST, None, ()),
         'auth-cancel': (GObject.SignalFlags.RUN_LAST, None, ()),
         'auth-busy': (GObject.SignalFlags.RUN_LAST, None, (bool,)),
-        'auth-prompt': (GObject.SignalFlags.RUN_LAST, None, (str,))
+        'auth-prompt': (GObject.SignalFlags.RUN_LAST, None, (str,)),
+        'auth-info': (GObject.SignalFlags.RUN_LAST, None, (str,))
     }
 
     def __init__(self):
@@ -34,10 +36,13 @@ class AuthClient(GObject.Object):
         if self.initialized:
             return True
 
+        if status.Debug:
+            print("authClient: attempting to initialize")
+
         try:
             helper_path = None
             architecture = platform.machine()
-            paths = ["/usr/lib", "/usr/lib/cinnamon-screensaver", "/usr/libexec"]
+            paths = [config.libexecdir, "/usr/lib", "/usr/lib/cinnamon-screensaver", "/usr/libexec", "/usr/libexec/cinnamon-screensaver"]
 
             # On x86 archs, iterate through multiple paths
             # For instance, on a Mint i686 box, the path is actually /usr/lib/i386-linux-gnu
@@ -57,7 +62,7 @@ class AuthClient(GObject.Object):
                     break
 
             if helper_path is None:
-                print ("Critical Error: PAM Helper could not be found!")
+                print ("authClient: critical Error: PAM Helper could not be found!")
 
             if status.Debug:
                 argv = (helper_path, "--debug", None)
@@ -72,7 +77,7 @@ class AuthClient(GObject.Object):
                                                Gio.SubprocessFlags.STDERR_SILENCE)
 
         except GLib.Error as e:
-            print("error starting cinnamon-screensaver-pam-helper: %s" % e.message)
+            print("authClient: error starting cinnamon-screensaver-pam-helper: %s" % e.message)
             return False
 
         self.proc.wait_check_async(None, self.on_proc_completed, None)
@@ -84,13 +89,21 @@ class AuthClient(GObject.Object):
 
         self.initialized = True
 
+        if status.Debug:
+            print("authClient: initialized")
+
         return True
 
     def cancel(self):
         if self.proc != None:
             self.message_to_child("CS_PAM_AUTH_REQUEST_SUBPROCESS_EXIT\n");
+        else:
+            if status.Debug:
+                print("authClient: cancel requested, but no helper process")
 
     def on_proc_completed(self, proc, res, data=None):
+        if status.Debug:
+            print("authClient: helper process completed...")
         try:
             ret = proc.wait_check_finish(res)
         except GLib.Error as e:
@@ -122,9 +135,12 @@ class AuthClient(GObject.Object):
         if not self.initialized:
             return
 
+        if status.Debug:
+            print("authClient: message to child")
+
         try:
             b = GLib.Bytes.new(string.encode())
-            status = self.in_pipe.write_bytes(b)
+            s = self.in_pipe.write_bytes(b)
 
             self.in_pipe.flush(None)
         except GLib.Error as e:
@@ -159,20 +175,39 @@ class AuthClient(GObject.Object):
                     if "CS_PAM_AUTH_SET_PROMPT" in output:
                         prompt = re.search('(?<=CS_PAM_AUTH_SET_PROMPT_)(.*)(?=_)', output).group(0)
                         self.emit_idle_auth_prompt(prompt)
+                    if "CS_PAM_AUTH_SET_INFO" in output:
+                        info = re.search('(?<=CS_PAM_AUTH_SET_INFO_)(.*)(?=_)', output).group(0)
+                        self.emit_auth_info(info)
+
         if not finished:
             pipe.read_bytes_async(1024, GLib.PRIORITY_DEFAULT, None, self.message_from_child)
 
     def emit_idle_busy_state(self, busy):
+        if status.Debug:
+            print("authClient: idle add auth-busy")
         GObject.idle_add(self.emit, "auth-busy", busy)
 
     def emit_idle_failure(self):
+        if status.Debug:
+            print("authClient: idle add failure")
         GObject.idle_add(self.emit, "auth-failure")
 
     def emit_idle_success(self):
+        if status.Debug:
+            print("authClient: idle add success")
         GObject.idle_add(self.emit, "auth-success")
 
     def emit_idle_cancel(self):
+        if status.Debug:
+            print("authClient: idle add cancel")
         GObject.idle_add(self.emit, "auth-cancel")
 
     def emit_idle_auth_prompt(self, prompt):
+        if status.Debug:
+            print("authClient: idle add auth-prompt")
         GObject.idle_add(self.emit, "auth-prompt", prompt)
+
+    def emit_auth_info(self, info):
+        if status.Debug:
+            print("authClient: auth-info")
+        GObject.idle_add(self.emit, "auth-info", info)
