@@ -66,6 +66,7 @@ class Stage(Gtk.Window):
         self.unlock_dialog = None
         self.audio_panel = None
         self.info_panel = None
+        self.osk = None
 
         self.stage_refresh_id = 0
 
@@ -116,7 +117,7 @@ class Stage(Gtk.Window):
         # This filter suppresses any other windows that might share
         # our window group in muffin, from showing up over the Stage.
         # For instance: Chrome and Firefox native notifications.
-        self.gdk_filter = CScreensaver.GdkEventFilter()
+        self.gdk_filter = CScreensaver.GdkEventFilter.new(self, 0)
 
         trackers.con_tracker_get().connect(status.screen,
                                            "size-changed",
@@ -180,6 +181,7 @@ class Stage(Gtk.Window):
 
     def on_composited_changed(self, screen, data=None):
         if self.get_realized():
+            self.manager.kill_fallback_window()
 
             user_time = self.get_display().get_user_time()
 
@@ -191,6 +193,9 @@ class Stage(Gtk.Window):
             self.get_window().set_user_time(user_time)
             self.show()
 
+            if status.Locked:
+                self.manager.spawn_fallback_window()
+
             GObject.idle_add(self.manager.grab_stage)
 
     def on_grab_broken_event(self, widget, event, data=None):
@@ -257,7 +262,7 @@ class Stage(Gtk.Window):
 
         self.setup_children()
 
-        self.gdk_filter.start(self)
+        self.gdk_filter.start()
 
         trackers.con_tracker_get().disconnect(self.overlay,
                                               "realize",
@@ -307,29 +312,32 @@ class Stage(Gtk.Window):
                 print("Problem setting up clock widget: %s" % str(e))
                 self.clock_widget = None
 
-            try:
-                self.setup_albumart()
-            except Exception as e:
-                print("Problem setting up albumart widget: %s" % str(e))
-                self.albumart_widget = None
-
-            try:
-                self.setup_status_bars()
-            except Exception as e:
-                print("Problem setting up status bars: %s" % str(e))
-                self.audio_panel = None
-                self.info_panel = None
-
             try:
                 self.setup_osk()
             except Exception as e:
                 print("Problem setting up on-screen keyboard: %s" % str(e))
                 self.osk = None
 
-        if total_failure:
+            trackers.timer_tracker_get().start("setup-delayed-components",
+                                               2000,
+                                               self.setup_delayed_components)
+        else:
             print("Total failure somewhere, deactivating screensaver.")
             GObject.idle_add(self.deactivate_after_timeout)
 
+    def setup_delayed_components(self, data=None):
+        try:
+            self.setup_albumart()
+        except Exception as e:
+            print("Problem setting up albumart widget: %s" % str(e))
+            self.albumart_widget = None
+        try:
+            self.setup_status_bars()
+        except Exception as e:
+            print("Problem setting up status bars: %s" % str(e))
+            self.audio_panel = None
+            self.info_panel = None
+
     def destroy_children(self):
         try:
             self.destroy_monitor_views()
@@ -404,6 +412,7 @@ class Stage(Gtk.Window):
 
         self.set_timeout_active(None, False)
 
+        trackers.timer_tracker_get().cancel("setup-delayed-components")
         self.destroy_children()
 
         self.gdk_filter.stop()
@@ -479,7 +488,8 @@ class Stage(Gtk.Window):
         if status.Debug:
             print("stage: Power state changed, updating info panel")
 
-        self.info_panel.update_visibility()
+        if self.info_panel != None:
+            self.info_panel.update_visibility()
 
     def setup_clock(self):
         """
@@ -510,7 +520,7 @@ class Stage(Gtk.Window):
         self.albumart_widget = AlbumArt(None, status.screen.get_mouse_monitor())
         self.add_child_widget(self.albumart_widget)
 
-        self.floaters.append(self.clock_widget)
+        self.floaters.append(self.albumart_widget)
 
         if settings.get_show_albumart():
             self.albumart_widget.start_positioning()
@@ -565,10 +575,13 @@ class Stage(Gtk.Window):
         self.audio_panel = AudioPanel()
         self.add_child_widget(self.audio_panel)
 
+        if status.Awake:
+            self.audio_panel.show_panel()
+
         self.info_panel = InfoPanel()
         self.add_child_widget(self.info_panel)
 
-        self.info_panel.update_visibility()
+        self.info_panel.refresh_power_state()
 
     def queue_dialog_key_event(self, event):
         """
@@ -672,7 +685,7 @@ class Stage(Gtk.Window):
         if self.audio_panel != None:
             self.audio_panel.show_panel()
         if self.info_panel != None:
-            self.info_panel.update_visibility()
+            self.info_panel.refresh_power_state()
         if self.osk != None:
             self.osk.show()
 
@@ -707,7 +720,7 @@ class Stage(Gtk.Window):
         status.Awake = False
 
         self.update_monitor_views()
-        self.info_panel.update_visibility()
+        self.info_panel.refresh_power_state()
 
     def update_monitor_views(self):
         """
